/// @file
////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2017/18      Christian Lessig, Otto-von-Guericke Universitaet Magdeburg
///
////////////////////////////////////////////////////////////////////////////////////////////////////
///
///  module     : tutorial 4
///
///  author     : lessig@isg.cs.ovgu.de
///
///  project    : GPU Programming
///
///  description: race conditions
///
////////////////////////////////////////////////////////////////////////////////////////////////////

// includes, file
#include "dnsserver.h"


/*static*/ DNSServer* DNSServer::instance = nullptr;

////////////////////////////////////////////////////////////////////////////////////////////////////
//! Static construction interface
////////////////////////////////////////////////////////////////////////////////////////////////////
/*static*/
DNSServer*
DNSServer::get() {

  if( nullptr == instance) {
      instance = new DNSServer();
  }

  return instance;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//! Constructor, default
////////////////////////////////////////////////////////////////////////////////////////////////////
DNSServer::DNSServer() :
  data(),
  data_mutex()
{
  // some simple entries
  data.insert( DataEntry( "www.ovgu.de", 141447111));
  data.insert( DataEntry( "www.tu-berlin.de", 1301497201));
  data.insert( DataEntry( "www.caltech.edu", 21222310314));
  data.insert( DataEntry( "www.toronto.edu", 128100166120));
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//! Get DNS entry
////////////////////////////////////////////////////////////////////////////////////////////////////
DNSServer::DNSEntry
DNSServer::getDNS( const std::string& domain) const {

  // ensure safe ready access
  std::shared_lock<std::shared_timed_mutex> lk( data_mutex);

  // return value
  DNSEntry ret = -1;

  auto item = data.find( domain);
  if( item != data.end()) {
    ret = item->second;
  }

  return ret;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//! Get DNS entry
////////////////////////////////////////////////////////////////////////////////////////////////////
void
DNSServer::newEntry( const std::string& domain, const DNSEntry& dns) {

  // ensure safe ready access
  std::unique_lock<std::shared_timed_mutex> lk( data_mutex);

  // return value
  data.insert( DataEntry( domain, dns));
}
